/*
 *		Copyright (C) 2013-2015, 2019 by the Konclude Developer Team.
 *
 *		This file is part of the reasoning system Konclude.
 *		For details and support, see <http://konclude.com/>.
 *
 *		Konclude is free software: you can redistribute it and/or modify
 *		it under the terms of version 3 of the GNU Lesser General Public
 *		License (LGPLv3) as published by the Free Software Foundation.
 *
 *		Konclude is distributed in the hope that it will be useful,
 *		but WITHOUT ANY WARRANTY; without even the implied warranty of
 *		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *		GNU (Lesser) General Public License for more details.
 *
 *		You should have received a copy of the GNU (Lesser) General Public
 *		License along with Konclude. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "CThroughputTestTaskTester.h"


namespace Konclude {

	namespace Test {



		CThroughputTestTaskTester::CThroughputTestTaskTester() {
			handleAlg = new CThroughputTestTaskHandleAlgorithm();
		}


		CThroughputTestTaskTester::~CThroughputTestTaskTester() {
		}


		void CThroughputTestTaskTester::generateTestingStructure(cint64 maxBranchDepth, cint64 branchingFactor) {
			mMaxBranchDepth = maxBranchDepth;
			mBranchingFactor = branchingFactor;
			mProcessUnit = new CSingleThreadTaskProcessorUnit(handleAlg);
			mProcessUnit->startProcessing();
		}

		void CThroughputTestTaskTester::startTesting() {
			CThroughputTestTask* rootTask = CObjectMemoryPoolAllocator<CThroughputTestTask>::allocateAndConstructWithMemroyPool();
			rootTask->initTestTask(mMaxBranchDepth,mBranchingFactor);
			CTaskEventCommunicator::postSendTaskProcessEvent(mProcessUnit->getEventHandler(),rootTask,false,nullptr);
		}

		void CThroughputTestTaskTester::stopTesting() {
		}

		void CThroughputTestTaskTester::destroyTestingStructure() {
			mProcessUnit->stopProcessing();
			mProcessUnit->stopThread();
			delete mProcessUnit;
		}


		cint64 CThroughputTestTaskTester::getTaskProcessedCount() {
			cint64 totalProcessedCount = mProcessUnit->getTaskProcessingStatistics()->getStatisticTasksProcessedCount();
			return totalProcessedCount;
		}



	}; // end namespace Test

}; // end namespace Konclude
