/*
 *		Copyright (C) 2013-2015, 2019 by the Konclude Developer Team.
 *
 *		This file is part of the reasoning system Konclude.
 *		For details and support, see <http://konclude.com/>.
 *
 *		Konclude is free software: you can redistribute it and/or modify
 *		it under the terms of version 3 of the GNU Lesser General Public
 *		License (LGPLv3) as published by the Free Software Foundation.
 *
 *		Konclude is distributed in the hope that it will be useful,
 *		but WITHOUT ANY WARRANTY; without even the implied warranty of
 *		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *		GNU (Lesser) General Public License for more details.
 *
 *		You should have received a copy of the GNU (Lesser) General Public
 *		License along with Konclude. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "CDeletionContainer.h"


namespace Konclude {

	namespace Utilities {

		namespace Memory {


			CDeletionContainer::CDeletionContainer(CDeletionLinker* deletionLinker) {
				mDeletionLinker = deletionLinker;
			}

			CDeletionContainer* CDeletionContainer::addDeletionLinker(CDeletionLinker* deletionLinker) {
				if (deletionLinker) {
					mDeletionLinker = deletionLinker->append(mDeletionLinker);
				}
				return this;
			}

			CDeletionLinker* CDeletionContainer::getDeletionLinker() {
				return mDeletionLinker;
			}

			CDeletionLinker* CDeletionContainer::takeDeletionLinker() {
				CDeletionLinker* tmpMemoryPool = mDeletionLinker;
				mDeletionLinker = nullptr;
				return tmpMemoryPool;
			}


		}; // end namespace Memory

	}; // end namespace Utilities

}; // end namespace Konclude

