/*
 *		Copyright (C) 2013-2015, 2019 by the Konclude Developer Team.
 *
 *		This file is part of the reasoning system Konclude.
 *		For details and support, see <http://konclude.com/>.
 *
 *		Konclude is free software: you can redistribute it and/or modify
 *		it under the terms of version 3 of the GNU Lesser General Public
 *		License (LGPLv3) as published by the Free Software Foundation.
 *
 *		Konclude is distributed in the hope that it will be useful,
 *		but WITHOUT ANY WARRANTY; without even the implied warranty of
 *		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *		GNU (Lesser) General Public License for more details.
 *
 *		You should have received a copy of the GNU (Lesser) General Public
 *		License along with Konclude. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "CBackendRepresentativeMemoryCacheNominalIndividualIndirectConnectionData.h"


namespace Konclude {

	namespace Reasoner {

		namespace Kernel {

			namespace Cache {


				CBackendRepresentativeMemoryCacheNominalIndividualIndirectConnectionData::CBackendRepresentativeMemoryCacheNominalIndividualIndirectConnectionData() {
				}


				CBackendRepresentativeMemoryCacheNominalIndividualIndirectConnectionData* CBackendRepresentativeMemoryCacheNominalIndividualIndirectConnectionData::initNominalIndividualIndirectConnectionData(CBackendRepresentativeMemoryCacheNominalIndividualIndirectConnectionData* data) {
					mIndirectlyConnectedIndividualIdLinker = nullptr;
					mLastChangeId = 0;
					if (data) {
						mIndirectlyConnectedIndividualIdLinker = data->mIndirectlyConnectedIndividualIdLinker;
						mLastChangeId = data->mLastChangeId;
					}
					return this;
				}

				CXLinker<cint64>* CBackendRepresentativeMemoryCacheNominalIndividualIndirectConnectionData::getIndirectlyConnectedIndividualIdLinker() {
					return mIndirectlyConnectedIndividualIdLinker;
				}

				CBackendRepresentativeMemoryCacheNominalIndividualIndirectConnectionData* CBackendRepresentativeMemoryCacheNominalIndividualIndirectConnectionData::setIndirectlyConnectedIndividualIdLinker(CXLinker<cint64>* indirectlyConnectedIndividualIdLinker) {
					mIndirectlyConnectedIndividualIdLinker = indirectlyConnectedIndividualIdLinker;
					return this;
				}

				CBackendRepresentativeMemoryCacheNominalIndividualIndirectConnectionData* CBackendRepresentativeMemoryCacheNominalIndividualIndirectConnectionData::addIndirectlyConnectedIndividualIdLinker(CXLinker<cint64>* indirectlyConnectedIndividualIdLinker) {
					mIndirectlyConnectedIndividualIdLinker = indirectlyConnectedIndividualIdLinker->append(mIndirectlyConnectedIndividualIdLinker);
					return this;
				}
				
				cint64 CBackendRepresentativeMemoryCacheNominalIndividualIndirectConnectionData::getLastChangeId() {
					return mLastChangeId;
				}

				CBackendRepresentativeMemoryCacheNominalIndividualIndirectConnectionData* CBackendRepresentativeMemoryCacheNominalIndividualIndirectConnectionData::setLastChangeId(cint64 id) {
					return this;
				}




			}; // end namespace Cache

		}; // end namespace Kernel

	}; // end namespace Reasoner

}; // end namespace Konclude
