/*
 *		Copyright (C) 2013-2015, 2019 by the Konclude Developer Team.
 *
 *		This file is part of the reasoning system Konclude.
 *		For details and support, see <http://konclude.com/>.
 *
 *		Konclude is free software: you can redistribute it and/or modify
 *		it under the terms of version 3 of the GNU Lesser General Public
 *		License (LGPLv3) as published by the Free Software Foundation.
 *
 *		Konclude is distributed in the hope that it will be useful,
 *		but WITHOUT ANY WARRANTY; without even the implied warranty of
 *		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *		GNU (Lesser) General Public License for more details.
 *
 *		You should have received a copy of the GNU (Lesser) General Public
 *		License along with Konclude. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "COntologyTriplesData.h"


namespace Konclude {

	namespace Reasoner {

		namespace Ontology {


			COntologyTriplesData::COntologyTriplesData() {
				mLocLatestTriplesData = nullptr;
				mPrevLatestTriplesData = nullptr;
			}


			COntologyTriplesData* COntologyTriplesData::referenceTriplesData(COntologyTriplesData* tripleData) {
				mLocLatestTriplesData = nullptr;
				mPrevLatestTriplesData = nullptr;
				if (tripleData) {
					mPrevLatestTriplesData = tripleData->getLatestTriplesData();
					mTriplesDataStatus.referenceTriplesDataStatus(tripleData->getTriplesDataStatus());
					mAllTriplesDataList = tripleData->mAllTriplesDataList;
				}
				return this;
			}


			CTriplesData* COntologyTriplesData::getLatestTriplesData(bool forceLocal) {
				if (forceLocal || mLocLatestTriplesData) {
					return mLocLatestTriplesData;
				} else {
					return mPrevLatestTriplesData;
				}
			}

			QList<CTriplesData*>* COntologyTriplesData::getAllTriplesData() {
				return &mAllTriplesDataList;
			}



			COntologyTriplesData* COntologyTriplesData::addTriplesData(CTriplesData* triplesData) {
				if (triplesData != mLocLatestTriplesData) {
					mLocLatestTriplesData = triplesData;
					mAllTriplesDataList.append(triplesData);
				}
				return this;
			}

			COntologyTriplesDataStatus* COntologyTriplesData::getTriplesDataStatus() {
				return &mTriplesDataStatus;
			}


			COntologyTriplesAssertionsAccessor* COntologyTriplesData::getTripleAssertionAccessor() {
				return mTripleAssertionAccessor;
			}


			COntologyTriplesData* COntologyTriplesData::setTripleAssertionAccessor(COntologyTriplesAssertionsAccessor* accessor) {
				mTripleAssertionAccessor = accessor;
				return this;
			}



		}; // end namespace Ontology

	}; // end namespace Reasoner

}; // end namespace Konclude
