/*
 *		Copyright (C) 2013-2015, 2019 by the Konclude Developer Team.
 *
 *		This file is part of the reasoning system Konclude.
 *		For details and support, see <http://konclude.com/>.
 *
 *		Konclude is free software: you can redistribute it and/or modify
 *		it under the terms of version 3 of the GNU Lesser General Public
 *		License (LGPLv3) as published by the Free Software Foundation.
 *
 *		Konclude is distributed in the hope that it will be useful,
 *		but WITHOUT ANY WARRANTY; without even the implied warranty of
 *		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *		GNU (Lesser) General Public License for more details.
 *
 *		You should have received a copy of the GNU (Lesser) General Public
 *		License along with Konclude. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "CCommanderManagerConfigType.h"


namespace Konclude {

	namespace Control {

		namespace Command {

			CCommanderManagerConfigType::CCommanderManagerConfigType(CCommanderManager *reasonerCommander) {
				rCommander = reasonerCommander;
			}


			CCommanderManagerConfigType::~CCommanderManagerConfigType() {
			}


			CCommanderManagerConfigType *CCommanderManagerConfigType::getTypeCopy() {
				return new CCommanderManagerConfigType(rCommander);
			}


			CCommanderManager *CCommanderManagerConfigType::getCommanderManager() {
				return rCommander;
			}

			CCommanderManagerConfigType *CCommanderManagerConfigType::setCommanderManager(CCommanderManager *reasonerCommander) {
				rCommander = reasonerCommander;
				return this;
			}




			bool CCommanderManagerConfigType::equals(CConfigType *dataType) {
				CCommanderManagerConfigType *rCommanderType = dynamic_cast<CCommanderManagerConfigType *>(dataType);
				if (rCommanderType) {
					return (rCommander == rCommanderType->rCommander);
				}
				return false;
			}



		}; // end namespace Command

	}; // end namespace Control

}; // end namespace Konclude
