/*
 *		Copyright (C) 2013-2015, 2019 by the Konclude Developer Team.
 *
 *		This file is part of the reasoning system Konclude.
 *		For details and support, see <http://konclude.com/>.
 *
 *		Konclude is free software: you can redistribute it and/or modify
 *		it under the terms of version 3 of the GNU Lesser General Public
 *		License (LGPLv3) as published by the Free Software Foundation.
 *
 *		Konclude is distributed in the hope that it will be useful,
 *		but WITHOUT ANY WARRANTY; without even the implied warranty of
 *		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *		GNU (Lesser) General Public License for more details.
 *
 *		You should have received a copy of the GNU (Lesser) General Public
 *		License along with Konclude. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "CCallbackRememberLinkedExecuter.h"



namespace Konclude {

	namespace Concurrent {

		namespace Callback {



			CCallbackRememberLinkedExecuter::CCallbackRememberLinkedExecuter() {
				mCallbackLinker = nullptr;
				mCalledCallbackStop = nullptr;
				mCallbackExecuted = false;
			}


			CCallbackRememberLinkedExecuter::~CCallbackRememberLinkedExecuter() {
			}

			void CCallbackRememberLinkedExecuter::doCallback() {
				mCallbackExecuted = true;
				CCallbackData* callbackStop = mCalledCallbackStop;
				CCallbackData* newCallbackStop = mCallbackLinker;
				mCalledCallbackStop = newCallbackStop;
				CCallbackData* callbackIt = newCallbackStop;
				while (callbackIt && callbackIt != callbackStop) {
					CCallbackData* callback = callbackIt;
					callbackIt = callbackIt->getNext();
					callback->doCallback();
				}
			}


			qint64 CCallbackRememberLinkedExecuter::getCallbackCount() {
				cint64 count = 0;
				if (mCallbackLinker) {
					count = mCallbackLinker->getCount();
				}
				return count;
			}



			CCallbackRememberLinkedExecuter* CCallbackRememberLinkedExecuter::addCallbacks(CCallbackData* callbackData) {
				if (mCallbackExecuted) {
					CCallbackData* callbackIt = callbackData;
					while (callbackIt) {
						CCallbackData* callback = callbackIt;
						callbackIt = callbackIt->getNext();
						callback->doCallback();
					}
				} else {
					if (callbackData) {
						mCallbackLinker = callbackData->append(mCallbackLinker);
					}
					if (mCallbackExecuted) {
						doCallback();
					}
				}
				return this;
			}

			CCallbackRememberLinkedExecuter* CCallbackRememberLinkedExecuter::removeCallbacks(CCallbackData* callbackData) {
				CCallbackData* callbackIt = callbackData;
				while (callbackIt) {
					if (mCallbackLinker) {
						mCallbackLinker = mCallbackLinker->removeOne(callbackData);
					}
					callbackIt = callbackIt->getNext();
				}
				return this;
			}

		}; // end namespace Callback

	}; // end namespace Concurrent

}; // end namespace Konclude