/* Generated By: http&JavaCC: Do not edit this line. OWL2FunctionalStyleParserConstants.java */

/**
 * Token literal values and constants.
 * Generated by org.javacc.parser.OtherFilesGen#start()
 */
#ifndef OWL2FUNCTIONALSTYLEPARSERCONSTANTS_H
#define OWL2FUNCTIONALSTYLEPARSERCONSTANTS_H

namespace Konclude {
namespace Parser {
namespace FunctionalJAVACC {
  /** End of File. */
const  int _EOF = 0;
  /** RegularExpression Id. */
const  int OPEN_BRACKET = 5;
  /** RegularExpression Id. */
const  int CLOSE_BRACKET = 6;
  /** RegularExpression Id. */
const  int EQUALS = 7;
  /** RegularExpression Id. */
const  int REFERENCE = 8;
  /** RegularExpression Id. */
const  int PREFIX = 9;
  /** RegularExpression Id. */
const  int ONTOLOGY = 10;
  /** RegularExpression Id. */
const  int IMPORT = 11;
  /** RegularExpression Id. */
const  int CLASS = 12;
  /** RegularExpression Id. */
const  int DATATYPE = 13;
  /** RegularExpression Id. */
const  int OBJECT_PROPERTY = 14;
  /** RegularExpression Id. */
const  int DATA_PROPERTY = 15;
  /** RegularExpression Id. */
const  int ANNOTATION_PROPERTY = 16;
  /** RegularExpression Id. */
const  int NAMED_INDIVIDUAL = 17;
  /** RegularExpression Id. */
const  int DECLARATION = 18;
  /** RegularExpression Id. */
const  int OBJECT_INVERSE_OF = 19;
  /** RegularExpression Id. */
const  int OBJECT_PROPERTY_CHAIN = 20;
  /** RegularExpression Id. */
const  int COMPLEMENT_OF = 21;
  /** RegularExpression Id. */
const  int ONE_OF = 22;
  /** RegularExpression Id. */
const  int DATATYPE_RESTRICTION = 23;
  /** RegularExpression Id. */
const  int OBJECT_INTERSECTION_OF = 24;
  /** RegularExpression Id. */
const  int OBJECT_UNION_OF = 25;
  /** RegularExpression Id. */
const  int OBJECT_COMPLEMENT_OF = 26;
  /** RegularExpression Id. */
const  int OBJECT_ONE_OF = 27;
  /** RegularExpression Id. */
const  int OBJECT_SOME_VALUES_FROM = 28;
  /** RegularExpression Id. */
const  int OBJECT_ALL_VALUES_FROM = 29;
  /** RegularExpression Id. */
const  int OBJECT_HAS_VALUE = 30;
  /** RegularExpression Id. */
const  int OBJECT_HAS_SELF = 31;
  /** RegularExpression Id. */
const  int OBJECT_MIN_CARDINALITY = 32;
  /** RegularExpression Id. */
const  int OBJECT_MAX_CARDINALITY = 33;
  /** RegularExpression Id. */
const  int OBJECT_EXACT_CARDINALITY = 34;
  /** RegularExpression Id. */
const  int DATA_INTERSECTION_OF = 35;
  /** RegularExpression Id. */
const  int DATA_UNION_OF = 36;
  /** RegularExpression Id. */
const  int DATA_COMPLEMENT_OF = 37;
  /** RegularExpression Id. */
const  int DATA_ONE_OF = 38;
  /** RegularExpression Id. */
const  int DATA_SOME_VALUES_FROM = 39;
  /** RegularExpression Id. */
const  int DATA_ALL_VALUES_FROM = 40;
  /** RegularExpression Id. */
const  int DATA_HAS_VALUE = 41;
  /** RegularExpression Id. */
const  int DATA_MIN_CARDINALITY = 42;
  /** RegularExpression Id. */
const  int DATA_MAX_CARDINALITY = 43;
  /** RegularExpression Id. */
const  int DATA_EXACT_CARDINALITY = 44;
  /** RegularExpression Id. */
const  int SUB_CLASS_OF = 45;
  /** RegularExpression Id. */
const  int EQUIVALENT_CLASSES = 46;
  /** RegularExpression Id. */
const  int DISJOINT_CLASSES = 47;
  /** RegularExpression Id. */
const  int DISJOINT_UNION = 48;
  /** RegularExpression Id. */
const  int SUB_OBJECT_PROPERTY_OF = 49;
  /** RegularExpression Id. */
const  int EQUIVALENT_OBJECT_PROPERTIES = 50;
  /** RegularExpression Id. */
const  int DISJOINT_OBJECT_PROPERTIES = 51;
  /** RegularExpression Id. */
const  int OBJECT_PROPERTY_DOMAIN = 52;
  /** RegularExpression Id. */
const  int OBJECT_PROPERTY_RANGE = 53;
  /** RegularExpression Id. */
const  int INVERSE_OBJECT_PROPERTIES = 54;
  /** RegularExpression Id. */
const  int FUNCTIONAL_OBJECT_PROPERTY = 55;
  /** RegularExpression Id. */
const  int INVERSE_FUNCTIONAL_OBJECT_PROPERTY = 56;
  /** RegularExpression Id. */
const  int REFLEXIVE_OBJECT_PROPERTY = 57;
  /** RegularExpression Id. */
const  int IRREFLEXIVE_OBJECT_PROPERTY = 58;
  /** RegularExpression Id. */
const  int SYMMETRIC_OBJECT_PROPERTY = 59;
  /** RegularExpression Id. */
const  int ASYMMETRIC_OBJECT_PROPERTY = 60;
  /** RegularExpression Id. */
const  int TRANSITIVE_OBJECT_PROPERTY = 61;
  /** RegularExpression Id. */
const  int SUB_DATA_PROPERTY_OF = 62;
  /** RegularExpression Id. */
const  int EQUIVALENT_DATA_PROPERTIES = 63;
  /** RegularExpression Id. */
const  int DISJOINT_DATA_PROPERTIES = 64;
  /** RegularExpression Id. */
const  int DATA_PROPERTY_DOMAIN = 65;
  /** RegularExpression Id. */
const  int DATA_PROPERTY_RANGE = 66;
  /** RegularExpression Id. */
const  int FUNCTIONAL_DATA_PROPERTY = 67;
  /** RegularExpression Id. */
const  int DATATYPE_DEFINITION = 68;
  /** RegularExpression Id. */
const  int HAS_KEY = 69;
  /** RegularExpression Id. */
const  int SAME_INDIVIDUAL = 70;
  /** RegularExpression Id. */
const  int DIFFERENT_INDIVIDUALS = 71;
  /** RegularExpression Id. */
const  int CLASS_ASSERTION = 72;
  /** RegularExpression Id. */
const  int OBJECT_PROPERTY_ASSERTION = 73;
  /** RegularExpression Id. */
const  int NEGATIVE_OBJECT_PROPERTY_ASSERTION = 74;
  /** RegularExpression Id. */
const  int DATA_PROPERTY_ASSERTION = 75;
  /** RegularExpression Id. */
const  int NEGATIVE_DATA_PROPERTY_ASSERTION = 76;
  /** RegularExpression Id. */
const  int ANNOTATION = 77;
  /** RegularExpression Id. */
const  int ANNOTATION_ASSERTION = 78;
  /** RegularExpression Id. */
const  int SUB_ANNOTATION_PROPERTY_OF = 79;
  /** RegularExpression Id. */
const  int ANNOTATION_PROPERTY_DOMAIN = 80;
  /** RegularExpression Id. */
const  int ANNOTATION_PROPERTY_RANGE = 81;
  /** RegularExpression Id. */
const  int NON_NEGATIVE_INTEGER = 82;
  /** RegularExpression Id. */
const  int LANGTAG = 83;
  /** RegularExpression Id. */
const  int PNAME_NS = 84;
  /** RegularExpression Id. */
const  int PN_PREFIX = 85;
  /** RegularExpression Id. */
const  int BLANK_NODE_PREFIX = 86;
  /** RegularExpression Id. */
const  int DIGIT = 87;
  /** RegularExpression Id. */
const  int PN_CHARS_U = 88;
  /** RegularExpression Id. */
const  int PN_CHARS = 89;
  /** RegularExpression Id. */
const  int PN_CHARS_BASE = 90;
  /** RegularExpression Id. */
const  int FULL_IRI = 94;
  /** RegularExpression Id. */
const  int PN_LOCAL = 96;
  /** RegularExpression Id. */
const  int QUOTED_QString = 98;

  /** Lexical state. */
const  int DEFAULT = 0;
  /** Lexical state. */
const  int FullIri = 1;
  /** Lexical state. */
const  int FullIriEnd = 2;
  /** Lexical state. */
const  int AfterPrefix = 3;
  /** Lexical state. */
const  int QuotedQString = 4;
  /** Lexical state. */
const  int QuotedQStringEnd = 5;
  /** Lexical state. */
const  int Comment = 6;

  /** Literal token values. */
  static JAVACC_CHAR_TYPE tokenImage_arr_0[] = 
{0x3c, 0x45, 0x4f, 0x46, 0x3e, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_1[] = 
{0x22, 0x20, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_2[] = 
{0x22, 0x9, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_3[] = 
{0x22, 0xa, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_4[] = 
{0x22, 0xd, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_5[] = 
{0x22, 0x28, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_6[] = 
{0x22, 0x29, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_7[] = 
{0x22, 0x3d, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_8[] = 
{0x22, 0x5e, 0x5e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_9[] = 
{0x22, 0x50, 0x72, 0x65, 0x66, 0x69, 0x78, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_10[] = 
{0x22, 0x4f, 0x6e, 0x74, 0x6f, 0x6c, 0x6f, 0x67, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_11[] = 
{0x22, 0x49, 0x6d, 0x70, 0x6f, 0x72, 0x74, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_12[] = 
{0x22, 0x43, 0x6c, 0x61, 0x73, 0x73, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_13[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x74, 0x79, 0x70, 0x65, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_14[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_15[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_16[] = 
{0x22, 0x41, 0x6e, 0x6e, 0x6f, 0x74, 0x61, 0x74, 0x69, 0x6f, 0x6e, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_17[] = 
{0x22, 0x4e, 0x61, 0x6d, 0x65, 0x64, 0x49, 0x6e, 0x64, 0x69, 0x76, 0x69, 0x64, 0x75, 0x61, 0x6c, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_18[] = 
{0x22, 0x44, 0x65, 0x63, 0x6c, 0x61, 0x72, 0x61, 0x74, 0x69, 0x6f, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_19[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x49, 0x6e, 0x76, 0x65, 0x72, 0x73, 0x65, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_20[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x43, 0x68, 0x61, 0x69, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_21[] = 
{0x22, 0x43, 0x6f, 0x6d, 0x70, 0x6c, 0x65, 0x6d, 0x65, 0x6e, 0x74, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_22[] = 
{0x22, 0x4f, 0x6e, 0x65, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_23[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x74, 0x79, 0x70, 0x65, 0x52, 0x65, 0x73, 0x74, 0x72, 0x69, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_24[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x49, 0x6e, 0x74, 0x65, 0x72, 0x73, 0x65, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_25[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x55, 0x6e, 0x69, 0x6f, 0x6e, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_26[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x43, 0x6f, 0x6d, 0x70, 0x6c, 0x65, 0x6d, 0x65, 0x6e, 0x74, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_27[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x4f, 0x6e, 0x65, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_28[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x53, 0x6f, 0x6d, 0x65, 0x56, 0x61, 0x6c, 0x75, 0x65, 0x73, 0x46, 0x72, 0x6f, 0x6d, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_29[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x41, 0x6c, 0x6c, 0x56, 0x61, 0x6c, 0x75, 0x65, 0x73, 0x46, 0x72, 0x6f, 0x6d, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_30[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x48, 0x61, 0x73, 0x56, 0x61, 0x6c, 0x75, 0x65, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_31[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x48, 0x61, 0x73, 0x53, 0x65, 0x6c, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_32[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x4d, 0x69, 0x6e, 0x43, 0x61, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x6c, 0x69, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_33[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x4d, 0x61, 0x78, 0x43, 0x61, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x6c, 0x69, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_34[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x45, 0x78, 0x61, 0x63, 0x74, 0x43, 0x61, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x6c, 0x69, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_35[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x49, 0x6e, 0x74, 0x65, 0x72, 0x73, 0x65, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_36[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x55, 0x6e, 0x69, 0x6f, 0x6e, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_37[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x43, 0x6f, 0x6d, 0x70, 0x6c, 0x65, 0x6d, 0x65, 0x6e, 0x74, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_38[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x4f, 0x6e, 0x65, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_39[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x53, 0x6f, 0x6d, 0x65, 0x56, 0x61, 0x6c, 0x75, 0x65, 0x73, 0x46, 0x72, 0x6f, 0x6d, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_40[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x41, 0x6c, 0x6c, 0x56, 0x61, 0x6c, 0x75, 0x65, 0x73, 0x46, 0x72, 0x6f, 0x6d, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_41[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x48, 0x61, 0x73, 0x56, 0x61, 0x6c, 0x75, 0x65, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_42[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x4d, 0x69, 0x6e, 0x43, 0x61, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x6c, 0x69, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_43[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x4d, 0x61, 0x78, 0x43, 0x61, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x6c, 0x69, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_44[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x45, 0x78, 0x61, 0x63, 0x74, 0x43, 0x61, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x6c, 0x69, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_45[] = 
{0x22, 0x53, 0x75, 0x62, 0x43, 0x6c, 0x61, 0x73, 0x73, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_46[] = 
{0x22, 0x45, 0x71, 0x75, 0x69, 0x76, 0x61, 0x6c, 0x65, 0x6e, 0x74, 0x43, 0x6c, 0x61, 0x73, 0x73, 0x65, 0x73, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_47[] = 
{0x22, 0x44, 0x69, 0x73, 0x6a, 0x6f, 0x69, 0x6e, 0x74, 0x43, 0x6c, 0x61, 0x73, 0x73, 0x65, 0x73, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_48[] = 
{0x22, 0x44, 0x69, 0x73, 0x6a, 0x6f, 0x69, 0x6e, 0x74, 0x55, 0x6e, 0x69, 0x6f, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_49[] = 
{0x22, 0x53, 0x75, 0x62, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_50[] = 
{0x22, 0x45, 0x71, 0x75, 0x69, 0x76, 0x61, 0x6c, 0x65, 0x6e, 0x74, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x69, 0x65, 0x73, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_51[] = 
{0x22, 0x44, 0x69, 0x73, 0x6a, 0x6f, 0x69, 0x6e, 0x74, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x69, 0x65, 0x73, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_52[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x44, 0x6f, 0x6d, 0x61, 0x69, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_53[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x52, 0x61, 0x6e, 0x67, 0x65, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_54[] = 
{0x22, 0x49, 0x6e, 0x76, 0x65, 0x72, 0x73, 0x65, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x69, 0x65, 0x73, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_55[] = 
{0x22, 0x46, 0x75, 0x6e, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_56[] = 
{0x22, 0x49, 0x6e, 0x76, 0x65, 0x72, 0x73, 0x65, 0x46, 0x75, 0x6e, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_57[] = 
{0x22, 0x52, 0x65, 0x66, 0x6c, 0x65, 0x78, 0x69, 0x76, 0x65, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_58[] = 
{0x22, 0x49, 0x72, 0x72, 0x65, 0x66, 0x6c, 0x65, 0x78, 0x69, 0x76, 0x65, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_59[] = 
{0x22, 0x53, 0x79, 0x6d, 0x6d, 0x65, 0x74, 0x72, 0x69, 0x63, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_60[] = 
{0x22, 0x41, 0x73, 0x79, 0x6d, 0x6d, 0x65, 0x74, 0x72, 0x69, 0x63, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_61[] = 
{0x22, 0x54, 0x72, 0x61, 0x6e, 0x73, 0x69, 0x74, 0x69, 0x76, 0x65, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_62[] = 
{0x22, 0x53, 0x75, 0x62, 0x44, 0x61, 0x74, 0x61, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_63[] = 
{0x22, 0x45, 0x71, 0x75, 0x69, 0x76, 0x61, 0x6c, 0x65, 0x6e, 0x74, 0x44, 0x61, 0x74, 0x61, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x69, 0x65, 0x73, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_64[] = 
{0x22, 0x44, 0x69, 0x73, 0x6a, 0x6f, 0x69, 0x6e, 0x74, 0x44, 0x61, 0x74, 0x61, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x69, 0x65, 0x73, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_65[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x44, 0x6f, 0x6d, 0x61, 0x69, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_66[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x52, 0x61, 0x6e, 0x67, 0x65, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_67[] = 
{0x22, 0x46, 0x75, 0x6e, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x44, 0x61, 0x74, 0x61, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_68[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x74, 0x79, 0x70, 0x65, 0x44, 0x65, 0x66, 0x69, 0x6e, 0x69, 0x74, 0x69, 0x6f, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_69[] = 
{0x22, 0x48, 0x61, 0x73, 0x4b, 0x65, 0x79, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_70[] = 
{0x22, 0x53, 0x61, 0x6d, 0x65, 0x49, 0x6e, 0x64, 0x69, 0x76, 0x69, 0x64, 0x75, 0x61, 0x6c, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_71[] = 
{0x22, 0x44, 0x69, 0x66, 0x66, 0x65, 0x72, 0x65, 0x6e, 0x74, 0x49, 0x6e, 0x64, 0x69, 0x76, 0x69, 0x64, 0x75, 0x61, 0x6c, 0x73, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_72[] = 
{0x22, 0x43, 0x6c, 0x61, 0x73, 0x73, 0x41, 0x73, 0x73, 0x65, 0x72, 0x74, 0x69, 0x6f, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_73[] = 
{0x22, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x41, 0x73, 0x73, 0x65, 0x72, 0x74, 0x69, 0x6f, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_74[] = 
{0x22, 0x4e, 0x65, 0x67, 0x61, 0x74, 0x69, 0x76, 0x65, 0x4f, 0x62, 0x6a, 0x65, 0x63, 0x74, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x41, 0x73, 0x73, 0x65, 0x72, 0x74, 0x69, 0x6f, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_75[] = 
{0x22, 0x44, 0x61, 0x74, 0x61, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x41, 0x73, 0x73, 0x65, 0x72, 0x74, 0x69, 0x6f, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_76[] = 
{0x22, 0x4e, 0x65, 0x67, 0x61, 0x74, 0x69, 0x76, 0x65, 0x44, 0x61, 0x74, 0x61, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x41, 0x73, 0x73, 0x65, 0x72, 0x74, 0x69, 0x6f, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_77[] = 
{0x22, 0x41, 0x6e, 0x6e, 0x6f, 0x74, 0x61, 0x74, 0x69, 0x6f, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_78[] = 
{0x22, 0x41, 0x6e, 0x6e, 0x6f, 0x74, 0x61, 0x74, 0x69, 0x6f, 0x6e, 0x41, 0x73, 0x73, 0x65, 0x72, 0x74, 0x69, 0x6f, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_79[] = 
{0x22, 0x53, 0x75, 0x62, 0x41, 0x6e, 0x6e, 0x6f, 0x74, 0x61, 0x74, 0x69, 0x6f, 0x6e, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x4f, 0x66, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_80[] = 
{0x22, 0x41, 0x6e, 0x6e, 0x6f, 0x74, 0x61, 0x74, 0x69, 0x6f, 0x6e, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x44, 0x6f, 0x6d, 0x61, 0x69, 0x6e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_81[] = 
{0x22, 0x41, 0x6e, 0x6e, 0x6f, 0x74, 0x61, 0x74, 0x69, 0x6f, 0x6e, 0x50, 0x72, 0x6f, 0x70, 0x65, 0x72, 0x74, 0x79, 0x52, 0x61, 0x6e, 0x67, 0x65, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_82[] = 
{0x22, 0x3c, 0x4e, 0x4f, 0x4e, 0x5f, 0x4e, 0x45, 0x47, 0x41, 0x54, 0x49, 0x56, 0x45, 0x5f, 0x49, 0x4e, 0x54, 0x45, 0x47, 0x45, 0x52, 0x3e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_83[] = 
{0x22, 0x3c, 0x4c, 0x41, 0x4e, 0x47, 0x54, 0x41, 0x47, 0x3e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_84[] = 
{0x22, 0x3c, 0x50, 0x4e, 0x41, 0x4d, 0x45, 0x5f, 0x4e, 0x53, 0x3e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_85[] = 
{0x22, 0x3c, 0x50, 0x4e, 0x5f, 0x50, 0x52, 0x45, 0x46, 0x49, 0x58, 0x3e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_86[] = 
{0x22, 0x5f, 0x3a, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_87[] = 
{0x22, 0x3c, 0x44, 0x49, 0x47, 0x49, 0x54, 0x3e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_88[] = 
{0x22, 0x3c, 0x50, 0x4e, 0x5f, 0x43, 0x48, 0x41, 0x52, 0x53, 0x5f, 0x55, 0x3e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_89[] = 
{0x22, 0x3c, 0x50, 0x4e, 0x5f, 0x43, 0x48, 0x41, 0x52, 0x53, 0x3e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_90[] = 
{0x22, 0x3c, 0x50, 0x4e, 0x5f, 0x43, 0x48, 0x41, 0x52, 0x53, 0x5f, 0x42, 0x41, 0x53, 0x45, 0x3e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_91[] = 
{0x22, 0x3c, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_92[] = 
{0x22, 0x22, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_93[] = 
{0x22, 0x23, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_94[] = 
{0x22, 0x3c, 0x46, 0x55, 0x4c, 0x4c, 0x5f, 0x49, 0x52, 0x49, 0x3e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_95[] = 
{0x22, 0x3e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_96[] = 
{0x22, 0x3c, 0x50, 0x4e, 0x5f, 0x4c, 0x4f, 0x43, 0x41, 0x4c, 0x3e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_97[] = 
{0x22, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_98[] = 
{0x22, 0x3c, 0x51, 0x55, 0x4f, 0x54, 0x45, 0x44, 0x5f, 0x51, 0x53, 0x74, 0x72, 0x69, 0x6e, 0x67, 0x3e, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_99[] = 
{0x22, 0x22, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_100[] = 
{0x22, 0xa, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_101[] = 
{0x22, 0xd, 0x22, 0};
  static JAVACC_CHAR_TYPE tokenImage_arr_102[] = 
{0x22, 0x3c, 0x74, 0x6f, 0x6b, 0x65, 0x6e, 0x20, 0x6f, 0x66, 0x20, 0x6b, 0x69, 0x6e, 0x64, 0x20, 0x31, 0x30, 0x32, 0x3e, 0x22, 0};
  static JAVACC_STRING_TYPE tokenImage[] = {
tokenImage_arr_0, 
tokenImage_arr_1, 
tokenImage_arr_2, 
tokenImage_arr_3, 
tokenImage_arr_4, 
tokenImage_arr_5, 
tokenImage_arr_6, 
tokenImage_arr_7, 
tokenImage_arr_8, 
tokenImage_arr_9, 
tokenImage_arr_10, 
tokenImage_arr_11, 
tokenImage_arr_12, 
tokenImage_arr_13, 
tokenImage_arr_14, 
tokenImage_arr_15, 
tokenImage_arr_16, 
tokenImage_arr_17, 
tokenImage_arr_18, 
tokenImage_arr_19, 
tokenImage_arr_20, 
tokenImage_arr_21, 
tokenImage_arr_22, 
tokenImage_arr_23, 
tokenImage_arr_24, 
tokenImage_arr_25, 
tokenImage_arr_26, 
tokenImage_arr_27, 
tokenImage_arr_28, 
tokenImage_arr_29, 
tokenImage_arr_30, 
tokenImage_arr_31, 
tokenImage_arr_32, 
tokenImage_arr_33, 
tokenImage_arr_34, 
tokenImage_arr_35, 
tokenImage_arr_36, 
tokenImage_arr_37, 
tokenImage_arr_38, 
tokenImage_arr_39, 
tokenImage_arr_40, 
tokenImage_arr_41, 
tokenImage_arr_42, 
tokenImage_arr_43, 
tokenImage_arr_44, 
tokenImage_arr_45, 
tokenImage_arr_46, 
tokenImage_arr_47, 
tokenImage_arr_48, 
tokenImage_arr_49, 
tokenImage_arr_50, 
tokenImage_arr_51, 
tokenImage_arr_52, 
tokenImage_arr_53, 
tokenImage_arr_54, 
tokenImage_arr_55, 
tokenImage_arr_56, 
tokenImage_arr_57, 
tokenImage_arr_58, 
tokenImage_arr_59, 
tokenImage_arr_60, 
tokenImage_arr_61, 
tokenImage_arr_62, 
tokenImage_arr_63, 
tokenImage_arr_64, 
tokenImage_arr_65, 
tokenImage_arr_66, 
tokenImage_arr_67, 
tokenImage_arr_68, 
tokenImage_arr_69, 
tokenImage_arr_70, 
tokenImage_arr_71, 
tokenImage_arr_72, 
tokenImage_arr_73, 
tokenImage_arr_74, 
tokenImage_arr_75, 
tokenImage_arr_76, 
tokenImage_arr_77, 
tokenImage_arr_78, 
tokenImage_arr_79, 
tokenImage_arr_80, 
tokenImage_arr_81, 
tokenImage_arr_82, 
tokenImage_arr_83, 
tokenImage_arr_84, 
tokenImage_arr_85, 
tokenImage_arr_86, 
tokenImage_arr_87, 
tokenImage_arr_88, 
tokenImage_arr_89, 
tokenImage_arr_90, 
tokenImage_arr_91, 
tokenImage_arr_92, 
tokenImage_arr_93, 
tokenImage_arr_94, 
tokenImage_arr_95, 
tokenImage_arr_96, 
tokenImage_arr_97, 
tokenImage_arr_98, 
tokenImage_arr_99, 
tokenImage_arr_100, 
tokenImage_arr_101, 
tokenImage_arr_102, 
  };

}
}
}
#endif
