/*
 *		Copyright (C) 2013-2015, 2019 by the Konclude Developer Team.
 *
 *		This file is part of the reasoning system Konclude.
 *		For details and support, see <http://konclude.com/>.
 *
 *		Konclude is free software: you can redistribute it and/or modify
 *		it under the terms of version 3 of the GNU Lesser General Public
 *		License (LGPLv3) as published by the Free Software Foundation.
 *
 *		Konclude is distributed in the hope that it will be useful,
 *		but WITHOUT ANY WARRANTY; without even the implied warranty of
 *		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *		GNU (Lesser) General Public License for more details.
 *
 *		You should have received a copy of the GNU (Lesser) General Public
 *		License along with Konclude. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "COptimizedKPSetRolePossibleSubsumptionData.h"
#include "COptimizedKPSetRoleTestingItem.h"


namespace Konclude {

	namespace Reasoner {

		namespace Classifier {


			COptimizedKPSetRolePossibleSubsumptionData::COptimizedKPSetRolePossibleSubsumptionData(COptimizedKPSetRoleTestingItem* item) {
				mSubsumptionUpdatedFlag = false;
				mSubsumptionConfirmedFlag = false;
				mSubsumptionInvalidFlag = false;
				mTestingItem = item;
			}

			bool COptimizedKPSetRolePossibleSubsumptionData::isUpdateRequired() {
				return !mSubsumptionUpdatedFlag && (mSubsumptionConfirmedFlag || mSubsumptionInvalidFlag);
			}

			bool COptimizedKPSetRolePossibleSubsumptionData::isSubsumptionUpdated() {
				return mSubsumptionUpdatedFlag;
			}

			COptimizedKPSetRolePossibleSubsumptionData* COptimizedKPSetRolePossibleSubsumptionData::setSubsumptionUpdated(bool updated) {
				mSubsumptionUpdatedFlag = updated;
				return this;
			}

			bool COptimizedKPSetRolePossibleSubsumptionData::isSubsumptionConfirmed() {
				return mSubsumptionConfirmedFlag;
			}

			COptimizedKPSetRolePossibleSubsumptionData* COptimizedKPSetRolePossibleSubsumptionData::setSubsumptionConfirmed(bool confirmedSubsumption) {
				mSubsumptionConfirmedFlag = confirmedSubsumption;
				return this;
			}

			bool COptimizedKPSetRolePossibleSubsumptionData::isSubsumptionInvalided() {
				return mSubsumptionInvalidFlag;
			}

			COptimizedKPSetRolePossibleSubsumptionData* COptimizedKPSetRolePossibleSubsumptionData::setSubsumptionInvalid(bool invalidSubsumption) {
				mSubsumptionInvalidFlag = invalidSubsumption;
				return this;
			}

			bool COptimizedKPSetRolePossibleSubsumptionData::isSubsumptionKnown() {
				return mSubsumptionInvalidFlag || mSubsumptionConfirmedFlag;
			}

			bool COptimizedKPSetRolePossibleSubsumptionData::isSubsumptionUnknown() {
				return !mSubsumptionConfirmedFlag && !mSubsumptionInvalidFlag;
			}

			COptimizedKPSetRoleTestingItem* COptimizedKPSetRolePossibleSubsumptionData::getTestingItem() {
				return mTestingItem;
			}

			COptimizedKPSetRolePossibleSubsumptionData* COptimizedKPSetRolePossibleSubsumptionData::setTestingItem(COptimizedKPSetRoleTestingItem* item) {
				mTestingItem = item;
				return this;
			}


		}; // end namespace Classifier

	}; // end namespace Reasoner

}; // end namespace Konclude
