/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class com_konclude_jnibridge_QueryingBridge */

#ifndef _Included_com_konclude_jnibridge_QueryingBridge
#define _Included_com_konclude_jnibridge_QueryingBridge
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     com_konclude_jnibridge_QueryingBridge
 * Method:    checkIsOWLOntologyConsistent
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)Z
 */
JNIEXPORT jboolean JNICALL Java_com_konclude_jnibridge_QueryingBridge_checkIsOWLOntologyConsistent
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_QueryingBridge
 * Method:    queryOWLSubClasses
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;Lcom/konclude/jnibridge/SetOfObjectSetCallbackListener;Z)V
 */
JNIEXPORT void JNICALL Java_com_konclude_jnibridge_QueryingBridge_queryOWLSubClasses
  (JNIEnv *, jobject, jobject, jstring, jobject, jobject, jboolean);

/*
 * Class:     com_konclude_jnibridge_QueryingBridge
 * Method:    queryOWLSuperClasses
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;Lcom/konclude/jnibridge/SetOfObjectSetCallbackListener;Z)V
 */
JNIEXPORT void JNICALL Java_com_konclude_jnibridge_QueryingBridge_queryOWLSuperClasses
  (JNIEnv *, jobject, jobject, jstring, jobject, jobject, jboolean);

/*
 * Class:     com_konclude_jnibridge_QueryingBridge
 * Method:    queryOWLEquivalentClasses
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;Lcom/konclude/jnibridge/ObjectSetCallbackListener;)V
 */
JNIEXPORT void JNICALL Java_com_konclude_jnibridge_QueryingBridge_queryOWLEquivalentClasses
  (JNIEnv *, jobject, jobject, jstring, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_QueryingBridge
 * Method:    queryOWLSubObjectProperties
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;Lcom/konclude/jnibridge/SetOfObjectSetCallbackListener;ZZ)V
 */
JNIEXPORT void JNICALL Java_com_konclude_jnibridge_QueryingBridge_queryOWLSubObjectProperties
  (JNIEnv *, jobject, jobject, jstring, jobject, jobject, jboolean, jboolean);

/*
 * Class:     com_konclude_jnibridge_QueryingBridge
 * Method:    queryOWLSuperObjectProperties
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;Lcom/konclude/jnibridge/SetOfObjectSetCallbackListener;ZZ)V
 */
JNIEXPORT void JNICALL Java_com_konclude_jnibridge_QueryingBridge_queryOWLSuperObjectProperties
  (JNIEnv *, jobject, jobject, jstring, jobject, jobject, jboolean, jboolean);

/*
 * Class:     com_konclude_jnibridge_QueryingBridge
 * Method:    queryOWLEquivalentObjectProperties
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;Lcom/konclude/jnibridge/ObjectSetCallbackListener;)V
 */
JNIEXPORT void JNICALL Java_com_konclude_jnibridge_QueryingBridge_queryOWLEquivalentObjectProperties
  (JNIEnv *, jobject, jobject, jstring, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_QueryingBridge
 * Method:    queryOWLInstances
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;Lcom/konclude/jnibridge/SetOfObjectSetCallbackListener;Z)V
 */
JNIEXPORT void JNICALL Java_com_konclude_jnibridge_QueryingBridge_queryOWLInstances
  (JNIEnv *, jobject, jobject, jstring, jobject, jobject, jboolean);

/*
 * Class:     com_konclude_jnibridge_QueryingBridge
 * Method:    queryOWLTypes
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;Lcom/konclude/jnibridge/SetOfObjectSetCallbackListener;Z)V
 */
JNIEXPORT void JNICALL Java_com_konclude_jnibridge_QueryingBridge_queryOWLTypes
  (JNIEnv *, jobject, jobject, jstring, jobject, jobject, jboolean);

/*
 * Class:     com_konclude_jnibridge_QueryingBridge
 * Method:    queryOWLSameIndividuals
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;Lcom/konclude/jnibridge/SetOfObjectSetCallbackListener;)V
 */
JNIEXPORT void JNICALL Java_com_konclude_jnibridge_QueryingBridge_queryOWLSameIndividuals
  (JNIEnv *, jobject, jobject, jstring, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_QueryingBridge
 * Method:    queryOWLObjectPropertySources
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;Ljava/lang/String;Ljava/lang/Object;Lcom/konclude/jnibridge/SetOfObjectSetCallbackListener;)V
 */
JNIEXPORT void JNICALL Java_com_konclude_jnibridge_QueryingBridge_queryOWLObjectPropertySources
  (JNIEnv *, jobject, jobject, jstring, jobject, jstring, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_QueryingBridge
 * Method:    queryOWLObjectPropertyTargets
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;Ljava/lang/String;Ljava/lang/Object;Lcom/konclude/jnibridge/SetOfObjectSetCallbackListener;)V
 */
JNIEXPORT void JNICALL Java_com_konclude_jnibridge_QueryingBridge_queryOWLObjectPropertyTargets
  (JNIEnv *, jobject, jobject, jstring, jobject, jstring, jobject, jobject);

#ifdef __cplusplus
}
#endif
#endif
