/*
 *		Copyright (C) 2013-2015, 2019 by the Konclude Developer Team.
 *
 *		This file is part of the reasoning system Konclude.
 *		For details and support, see <http://konclude.com/>.
 *
 *		Konclude is free software: you can redistribute it and/or modify
 *		it under the terms of version 3 of the GNU Lesser General Public
 *		License (LGPLv3) as published by the Free Software Foundation.
 *
 *		Konclude is distributed in the hope that it will be useful,
 *		but WITHOUT ANY WARRANTY; without even the implied warranty of
 *		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *		GNU (Lesser) General Public License for more details.
 *
 *		You should have received a copy of the GNU (Lesser) General Public
 *		License along with Konclude. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef KONCLUDE_REASONER_KERNEL_ALGORITHM_CINDEXEDINDIVIDUALASSERTIONCONVERTIONVISITOR_H
#define KONCLUDE_REASONER_KERNEL_ALGORITHM_CINDEXEDINDIVIDUALASSERTIONCONVERTIONVISITOR_H

// Libraries includes


// Namespace includes
#include "AlgorithmSettings.h"
#include "CCalculationAlgorithmContext.h"

// Other includes
#include "Context/CContextBase.h"

#include "Utilities/Container/CLinker.h"

#include "Reasoner/Preprocess/CPreProcessContextProviding.h"
#include "Reasoner/Preprocess/CDataLiteralNormalizerPreProcess.h"

#include "Reasoner/Ontology/COntologyTriplesIndividualAssertionsVisitor.h"
#include "Reasoner/Ontology/CConceptAssertionLinker.h"
#include "Reasoner/Ontology/CRoleAssertionLinker.h"
#include "Reasoner/Ontology/CReverseRoleAssertionLinker.h"
#include "Reasoner/Ontology/CDataAssertionLinker.h"
#include "Reasoner/Ontology/CIndividualProcessData.h"

// Logger includes
#include "Logger/CLogger.h"



namespace Konclude {

	using namespace Context;
	using namespace Utilities::Container;

	namespace Reasoner {

		using namespace Preprocess;
		using namespace Ontology;

		namespace Kernel {

			namespace Algorithm {

				/*! 
				 *
				 *		\class		CIndexedIndividualAssertionConvertionVisitor
				 *		\author		Andreas Steigmiller
				 *		\version	0.1
				 *		\brief		TODO
				 *
				 */
				class CIndexedIndividualAssertionConvertionVisitor : public COntologyTriplesIndividualAssertionsVisitor {
					// public methods
					public:
						//! Constructor
						CIndexedIndividualAssertionConvertionVisitor(cint64 retrievingIndiId, CCalculationAlgorithmContext* calcAlgContext);
						CIndexedIndividualAssertionConvertionVisitor(CIndividual* retrievingIndi, CCalculationAlgorithmContext* calcAlgContext);

						virtual bool visitIndividualName(const QString& indiName, COntologyTriplesAssertionsAccessor* accessor);


						virtual bool visitRoleAssertion(CRole* role, cint64 otherIndividualId, COntologyTriplesAssertionsAccessor* accessor);
						virtual bool visitReverseRoleAssertion(CRole* role, cint64 otherIndividualId, COntologyTriplesAssertionsAccessor* accessor);
						virtual bool visitDataAssertion(CRole* role, CDataLiteral* dataLiteral, COntologyTriplesAssertionsAccessor* accessor);
						virtual bool visitConceptAssertion(CConcept* concept, COntologyTriplesAssertionsAccessor* accessor);


						CConceptAssertionLinker* getConceptAssertionLinker();
						CRoleAssertionLinker* getRoleAssertionLinker();
						CReverseRoleAssertionLinker* getReverseRoleAssertionLinker();
						CDataAssertionLinker* getDataAssertionLinker();


						CIndividual* getRetrievalIndividual();

					// protected methods
					protected:
						CDataLiteralValue* createDataLiteralValue(CDataLiteral* dataLiteral);
						CIndividual* getExistingOrCreateNewTemporaryIndividual(cint64 indiId);
						CIndividual* getNewTemporaryIndividual(CIndividual* existingIndi);


					// protected variables
					protected:
						cint64 mRetrievingIndiId;
						CIndividual* mRetrievingIndi;

						CDataLiteralNormalizerPreProcess mDataLiteralNormalizer;
						CPreProcessContextProviding mPreprocessContext;
						CCalculationAlgorithmContext* mCalcAlgContext;
						CMemoryAllocationManager* mMemoryAllocationManager;


						CConceptAssertionLinker* mConceptAssertionLinker = nullptr;
						CRoleAssertionLinker* mRoleAssertionLinker = nullptr;
						CReverseRoleAssertionLinker* mReverseRoleAssertionLinker = nullptr;
						CDataAssertionLinker* mDataAssertionLinker = nullptr;


					// private methods
					private:

					// private variables
					private:

				};

			}; // end namespace Algorithm

		}; // end namespace Kernel

	}; // end namespace Reasoner

}; // end namespace Konclude

#endif // KONCLUDE_REASONER_KERNEL_ALGORITHM_CINDEXEDINDIVIDUALASSERTIONCONVERTIONVISITOR_H
