/*
 *		Copyright (C) 2013-2015, 2019 by the Konclude Developer Team.
 *
 *		This file is part of the reasoning system Konclude.
 *		For details and support, see <http://konclude.com/>.
 *
 *		Konclude is free software: you can redistribute it and/or modify
 *		it under the terms of version 3 of the GNU Lesser General Public
 *		License (LGPLv3) as published by the Free Software Foundation.
 *
 *		Konclude is distributed in the hope that it will be useful,
 *		but WITHOUT ANY WARRANTY; without even the implied warranty of
 *		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *		GNU (Lesser) General Public License for more details.
 *
 *		You should have received a copy of the GNU (Lesser) General Public
 *		License along with Konclude. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "COptimizedRepresentativeKPSetCombinedExistentialRoleSetCacheLabelItemData.h"


namespace Konclude {

	namespace Reasoner {

		namespace Realizer {


			COptimizedRepresentativeKPSetCombinedExistentialRoleSetCacheLabelItemData::COptimizedRepresentativeKPSetCombinedExistentialRoleSetCacheLabelItemData(CBackendRepresentativeMemoryLabelCacheItem* combinedExistentialRoleLabelCacheItem) {
				mCombinedExistentialRoleLabelCacheItem = combinedExistentialRoleLabelCacheItem;
				mNondeterministic = false;
			}


			bool COptimizedRepresentativeKPSetCombinedExistentialRoleSetCacheLabelItemData::isNondeterministic() {
				return mNondeterministic;
			}


			CBackendRepresentativeMemoryLabelCacheItem* COptimizedRepresentativeKPSetCombinedExistentialRoleSetCacheLabelItemData::getCombinedNeighbourRoleLabelCacheItem() {
				return mCombinedExistentialRoleLabelCacheItem;
			}


			COptimizedRepresentativeKPSetCombinedExistentialRoleSetCacheLabelItemData* COptimizedRepresentativeKPSetCombinedExistentialRoleSetCacheLabelItemData::setNondeterministic(bool possibleInstances) {
				mNondeterministic = possibleInstances;
				return this;
			}



			QSet<COptimizedKPSetRoleInstancesItem*>* COptimizedRepresentativeKPSetCombinedExistentialRoleSetCacheLabelItemData::getComplexCandiateInstancesItemSet(bool inverse) {
				if (inverse) {
					return &mInverseComplexCandiateInstancesItemSet;
				} else {
					return &mComplexCandiateInstancesItemSet;
				}
			}

			COptimizedRepresentativeKPSetCombinedExistentialRoleSetCacheLabelItemData* COptimizedRepresentativeKPSetCombinedExistentialRoleSetCacheLabelItemData::addComplexCandiateInstancesItem(COptimizedKPSetRoleInstancesItem* roleItem, bool inverse) {
				if (inverse) {
					mInverseComplexCandiateInstancesItemSet.insert(roleItem);
				} else {
					mComplexCandiateInstancesItemSet.insert(roleItem);
				}
				return this;
			}



			bool COptimizedRepresentativeKPSetCombinedExistentialRoleSetCacheLabelItemData::hasComplexCandiateInstancesItems() {
				return !mComplexCandiateInstancesItemSet.isEmpty() || !mInverseComplexCandiateInstancesItemSet.isEmpty();
			}


		}; // end namespace Realizer

	}; // end namespace Reasoner

}; // end namespace Konclude
