/*
 *		Copyright (C) 2013-2015, 2019 by the Konclude Developer Team.
 *
 *		This file is part of the reasoning system Konclude.
 *		For details and support, see <http://konclude.com/>.
 *
 *		Konclude is free software: you can redistribute it and/or modify
 *		it under the terms of version 3 of the GNU Lesser General Public
 *		License (LGPLv3) as published by the Free Software Foundation.
 *
 *		Konclude is distributed in the hope that it will be useful,
 *		but WITHOUT ANY WARRANTY; without even the implied warranty of
 *		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *		GNU (Lesser) General Public License for more details.
 *
 *		You should have received a copy of the GNU (Lesser) General Public
 *		License along with Konclude. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "CRealizationIndividualSorting.h"


namespace Konclude {

	namespace Reasoner {

		namespace Realization {


			CRealizationIndividualSorting::CRealizationIndividualSorting(SORTING_TYPE type, SORTING_ORDER order, SORTING_GROUPING grouping) {
				mType = type;
				mOrder = order;
				mGrouping = grouping;
			}

			CRealizationIndividualSorting::SORTING_TYPE CRealizationIndividualSorting::getType() const {
				return mType;
			}

			CRealizationIndividualSorting::SORTING_ORDER CRealizationIndividualSorting::getOrder() const {
				return mOrder;
			}

			CRealizationIndividualSorting::SORTING_GROUPING CRealizationIndividualSorting::getGrouping() const {
				return mGrouping;
			}


			bool CRealizationIndividualSorting::isNothing() const {
				return getType() == SORT_TYPE_NOTHING && getOrder() == SORT_ORDER_NOTHING && getGrouping() == SORT_GROUP_NOTHING;
			}



		}; // end namespace Realization

	}; // end namespace Reasoner

}; // end namespace Konclude
