/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class com_konclude_jnibridge_AxiomExpressionBuildingBridge */

#ifndef _Included_com_konclude_jnibridge_AxiomExpressionBuildingBridge
#define _Included_com_konclude_jnibridge_AxiomExpressionBuildingBridge
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLSubClassOfAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLSubClassOfAxiom
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLEquivalentClassesFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLEquivalentClassesFromArgumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLSubObjectPropertyOfAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLSubObjectPropertyOfAxiom
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLEquivalentObjectPropertiesFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLEquivalentObjectPropertiesFromArgumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLSubDataPropertyOfAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLSubDataPropertyOfAxiom
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLNegativeObjectPropertyAssertionAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLNegativeObjectPropertyAssertionAxiom
  (JNIEnv *, jobject, jobject, jlong, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLAnonymousIndividual
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLAnonymousIndividual
  (JNIEnv *, jobject, jobject, jstring, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLSubPropertyChainOfAxiomFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLSubPropertyChainOfAxiomFromArgumentList
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLSameIndividualAxiomFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLSameIndividualAxiomFromArgumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLInverseObjectPropertiesAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLInverseObjectPropertiesAxiom
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLInverseFunctionalObjectPropertyAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLInverseFunctionalObjectPropertyAxiom
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLIrreflexiveObjectPropertyAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLIrreflexiveObjectPropertyAxiom
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLTransitiveObjectPropertyAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLTransitiveObjectPropertyAxiom
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLClassAssertionAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLClassAssertionAxiom
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLSymmetricObjectPropertyAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLSymmetricObjectPropertyAxiom
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDeclarationAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDeclarationAxiom
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDisjointUnionAxiomFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDisjointUnionAxiomFromArgumentList
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLFunctionalObjectPropertyAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLFunctionalObjectPropertyAxiom
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLObjectPropertyAssertionAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLObjectPropertyAssertionAxiom
  (JNIEnv *, jobject, jobject, jlong, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLObjectPropertyRangeAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLObjectPropertyRangeAxiom
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDisjointObjectPropertiesAxiomFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDisjointObjectPropertiesAxiomFromArgumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDifferentIndividualsAxiomFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDifferentIndividualsAxiomFromArgumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLObjectPropertyDomainAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLObjectPropertyDomainAxiom
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDisjointClassesAxiomFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDisjointClassesAxiomFromArgumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLReflexiveObjectPropertyAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLReflexiveObjectPropertyAxiom
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLAsymmetricObjectPropertyAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLAsymmetricObjectPropertyAxiom
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLObjectOneOfFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLObjectOneOfFromArgumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLObjectHasSelf
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLObjectHasSelf
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLExactCardinality
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLExactCardinality
  (JNIEnv *, jobject, jobject, jlong, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLMaxCardinality
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLMaxCardinality
  (JNIEnv *, jobject, jobject, jlong, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLMinCardinality
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLMinCardinality
  (JNIEnv *, jobject, jobject, jlong, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLObjectHasValue
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLObjectHasValue
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLObjectInverseOf
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLObjectInverseOf
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLClass
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLClass
  (JNIEnv *, jobject, jobject, jstring, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLObjectProperty
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLObjectProperty
  (JNIEnv *, jobject, jobject, jstring, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLNamedIndividual
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLNamedIndividual
  (JNIEnv *, jobject, jobject, jstring, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLObjectIntersectionFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLObjectIntersectionFromArgumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLObjectUnionFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLObjectUnionFromArgumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLObjectComplement
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLObjectComplement
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLObjectSomeValuesFrom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLObjectSomeValuesFrom
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLObjectAllValuesFrom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLObjectAllValuesFrom
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    initArumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)V
 */
JNIEXPORT void JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_initArumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    closeArumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)V
 */
JNIEXPORT void JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_closeArumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    addArgument
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)V
 */
JNIEXPORT void JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_addArgument
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLLiteral
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JLjava/lang/String;Ljava/lang/Object;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLLiteral
  (JNIEnv *, jobject, jobject, jlong, jstring, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLFacetRestriction
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLFacetRestriction
  (JNIEnv *, jobject, jobject, jstring, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDataSomeValuesFrom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDataSomeValuesFrom
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDataAllValuesFrom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDataAllValuesFrom
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDataHasValue
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDataHasValue
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDataMinCardinality
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDataMinCardinality
  (JNIEnv *, jobject, jobject, jlong, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDataMaxCardinality
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDataMaxCardinality
  (JNIEnv *, jobject, jobject, jlong, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDataExactCardinality
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDataExactCardinality
  (JNIEnv *, jobject, jobject, jlong, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDataProperty
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDataProperty
  (JNIEnv *, jobject, jobject, jstring, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDataPropertyDomainAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDataPropertyDomainAxiom
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDataPropertyRangeAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDataPropertyRangeAxiom
  (JNIEnv *, jobject, jobject, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLNegativeDataPropertyAssertionAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLNegativeDataPropertyAssertionAxiom
  (JNIEnv *, jobject, jobject, jlong, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDataPropertyAssertionAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;JJJ)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDataPropertyAssertionAxiom
  (JNIEnv *, jobject, jobject, jlong, jlong, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDisjointDataPropertiesAxiomFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDisjointDataPropertiesAxiomFromArgumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLFunctionalDataPropertyAxiom
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLFunctionalDataPropertyAxiom
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLEquivalentDataPropertiesAxiomFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLEquivalentDataPropertiesAxiomFromArgumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDatatype
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;Ljava/lang/String;Ljava/lang/Object;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDatatype
  (JNIEnv *, jobject, jobject, jstring, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDataOneOfFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDataOneOfFromArgumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDataComplementOf
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDataComplementOf
  (JNIEnv *, jobject, jobject, jlong);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDataIntersectionOfFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDataIntersectionOfFromArgumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDataUnionOfFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDataUnionOfFromArgumentList
  (JNIEnv *, jobject, jobject);

/*
 * Class:     com_konclude_jnibridge_AxiomExpressionBuildingBridge
 * Method:    buildOWLDatatypeRestrictionFromArgumentList
 * Signature: (Lcom/konclude/jnibridge/KoncludeReasonerBridge;J)J
 */
JNIEXPORT jlong JNICALL Java_com_konclude_jnibridge_AxiomExpressionBuildingBridge_buildOWLDatatypeRestrictionFromArgumentList
  (JNIEnv *, jobject, jobject, jlong);

#ifdef __cplusplus
}
#endif
#endif
